package com.recordingblogs;

/**
 * An audio buffer.  In principle, the only reason we are using a separate
 * class is so that we can also keep track of the actual size of the buffer
 *
 * Copyright (c) 2003 Anton Kamenov.  This file and the included source code is the 
 * property of Anton Kamenov.  Under no circumstances will the file and source code be 
 * sold, transferred, modified, or commercially exploited in part or in whole, without 
 * the prior written consent of Anton Kamenov.
 */

public class AudioBuffer
{
	/**
	 * The buffer
	 */
    private byte [] m_buffer;
    
    /**
     * The size of data in the buffer, which could be smaller than the size
     * of the buffer.  For example, when reading wave files, at the end of the
     * file, the buffer may not be full
     */
    private int m_actualSize;

    /**
     * The one and only constructor
     */
    public AudioBuffer()
    {
        m_buffer = null;
        m_actualSize = 0;
    }

    /**
     * Get the size of the buffer
     * @return The size of the buffer
     */
    public int getSize()
    {
        return m_buffer.length;
    }

    /**
     * Set the size of the buffer.  Note that there are no checks for errors
     * in the argument
     * @param size The new size of the buffer
     */
    public void setSize(int size)
    {
        m_actualSize = size;
        m_buffer = new byte[size];
    }

    /**
     * Get the buffer
     * @return The buffer
     */
    public byte[] getBuffer()
    {
        return m_buffer;
    }

    /**
     * Get the actual size of data in the buffer
     * @return The actual size of data in the buffer
     */
    public int getActualSize()
    {
        return m_actualSize;
    }

    /**
     * Set the actual size of data in the buffer
     * @param size The actual size of data in the buffer
     */
    public void setActualSize(int size)
    {
        m_actualSize = size;
    }
}
