package com.recordingblogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.recordingblogs.oreffect.*;

/**
 * This dialog encompasses the JPanel that contains the controls for the effect
 * and contains a Close button.  The Close button, in addition to simply closing
 * the dialog, also tells the mixer to stop playing.
 * 
 * Copyright (c) 2003 Anton Kamenov.  This file and the included source code is the 
 * property of Anton Kamenov.  Under no circumstances will the file and source code be 
 * sold, transferred, modified, or commercially exploited in part or in whole, without 
 * the prior written consent of Anton Kamenov.
 */

public class EffectDialog extends JDialog implements ActionListener
{
	/**
	 * This serial version UID is not used 
	 */
	private static final long serialVersionUID = -2135421402263744777L;

	/**
	 * The Close button
	 */
	private JButton m_closeButton;
	
	/**
	 * The mixer.  It is needed so this dialog can tell the mixer to 
	 * stop playing when this dialog is closed
	 */
	private Mixer m_mixer;

	/**
	 * The one and only constructor
	 * @param panel The JPanel that contains the controls for the effect
	 * @param mixer The mixer that plays audio data and processes the effect
	 */
	public EffectDialog(EffectPanelInterface panel, Mixer mixer)
	{
        super();
        
		m_mixer = mixer;
		
        Action escape = new AbstractAction()
        {
        	static final long serialVersionUID = -1717679756166556296L;
        	
            @Override
			public void actionPerformed(ActionEvent e)
            {
    			m_mixer.stop();
                dispose();
            }
        };
        setTitle("Effect");		//$NON-NLS-1$
        setDefaultCloseOperation(DISPOSE_ON_CLOSE);
		
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints constraints = new GridBagConstraints();
		getContentPane().setLayout(layout);

		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.weightx = 1;
		constraints.weighty = 1;
		constraints.anchor = GridBagConstraints.NORTHWEST;
		constraints.fill = GridBagConstraints.BOTH;
		constraints.insets.left = 5;
		constraints.insets.top = 5;
		constraints.insets.bottom = 5;
		layout.setConstraints((JPanel) panel, constraints);
		getContentPane().add((JPanel) panel);

		m_closeButton = new JButton("Close");		//$NON-NLS-1$
		m_closeButton.addActionListener(this);
        m_closeButton.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke("ESCAPE"), 		//$NON-NLS-1$
        		"pressed ESCAPE");		//$NON-NLS-1$
        m_closeButton.getActionMap().put("pressed ESCAPE", escape);		//$NON-NLS-1$
		constraints.gridx = 1;
		constraints.weightx = 0;
		constraints.insets.right = 5;
		constraints.fill = GridBagConstraints.HORIZONTAL;
		layout.setConstraints(m_closeButton, constraints);
		getContentPane().add(m_closeButton);
		
		pack();
	}
	
	@Override
	public void actionPerformed(ActionEvent e)
	{
		if (e.getSource() == m_closeButton)
		{
			m_mixer.stop();
			dispose();
		}
	}
}
