package com.recordingblogs;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import javax.swing.JOptionPane;
import com.recordingblogs.oreffect.*;

/**
 * Test an effect JAR package
 * 
 * Copyright (c) 2003 Anton Kamenov.  This file and the included source code is the 
 * property of Anton Kamenov.  Under no circumstances will the file and source code be 
 * sold, transferred, modified, or commercially exploited in part or in whole, without 
 * the prior written consent of Anton Kamenov.
 */

public class ExampleDelayTest
{
	@SuppressWarnings("resource")
	public static void main(String[] args) 
	{
		// there are two reasons to declare a JFrame here.  First, the effects
		// in Orinj are loaded by a JFrame and we should be consistent just in
		// case.  Second, we need an actual object instantiated so that its
		// class loader can be used
		MainFrame frame = new MainFrame();
		frame.loadEffects();
        
        // list all effects in a dialog, in which the user can choose an
		// effect to test
        Vector<String> listdata = new Vector<>(0);
        for(int i = 0; i < EffectStore.m_store.size(); i++)
        	listdata.add(EffectStore.m_store.get(i).getTitle());
        SelectDialog selectdlg = new SelectDialog(listdata);
        selectdlg.setModal(true);
        selectdlg.setVisible(true);
        if (! selectdlg.getOK())
        	return;

        // choose a wave to play so that the effect can be tested
        WaveFileDialog wavedlg = new WaveFileDialog();
        wavedlg.setModal(true);
        wavedlg.setVisible(true);
        if (! wavedlg.getOK())
        	return;

        // open the chosen wave.  During opening, check whether the wave is 
        // of the right format (e.g., a RIFF file with a WAVE subchunk that
        // contains a format and a data chunk).  Opening the wave does not 
        // mean that any audio data will be processed.  The actual processing
        // of sound data happens below.
        // This test code works only on wave files with signed PCM encoding and 
        // with 44100 Hz sampling rate and 16-bit sampling resolution.  Thus, the 
        // WaveFile code will also check for these
        WaveFile wave = null;
        try
        {
        	wave = new WaveFile(wavedlg.getFileName().getText(), "r");		//$NON-NLS-1$
            if (! wave.openRead())
            {
	            System.out.println("Not the right wave file");				//$NON-NLS-1$
	            wave.close();
            	return;
            }
        }
        catch (IOException e)
        {
            System.out.println("Not the right wave file");					//$NON-NLS-1$
            System.out.println("Exception in WaveFile::openRead: " + e);	//$NON-NLS-1$
            return;
        }

        // create a mixer.  This mixer can only play and can only play one wave
        // and process one effect
        Mixer mixer = new Mixer();
        mixer.setWave(wave);

        // instantiate an effect and an effect panel (for the chosen effect)
        try
        {
        	// this is the chosen effect
			EffectStoreItem item = EffectStore.m_store.get(selectdlg.getSelected());
			
			// create the effect
			ClassLoader classloader = item.getClassLoader();
			Class<?> classEffect = classloader.loadClass(item.getEffectConstructor().getName());
			Object effect = classEffect.getConstructor((Class<?> []) null).newInstance((Object []) null);
			
			// create the panel for the effect, which needs two arguments
			Object [] arguments = new Object[2];
			arguments[0] = effect;
			arguments[1] = wave.getFormat();
			Class<?> classEffectPanel = classloader.loadClass(item.getDialogConstructor().getName());
			Class<?> [] cls = new Class[] { arguments[0].getClass(), arguments[1].getClass() };
			Object effectpanel = classEffectPanel.getConstructor(cls).newInstance(arguments);
			
			// put the effect panel in an actual dialog.  We need the dialog not only
			// to show the panel, but also so that we can use the dialog's Close button
			// to stop playback, so that we do not have to listen to the whole wave
			// all the time
	        EffectDialog effectdlg = new EffectDialog((EffectPanelInterface) effectpanel, mixer);
	        effectdlg.setModal(false);
	        effectdlg.setVisible(true);
	        
	        // tell the mixer which effect to use
	        mixer.setEffect((EffectInterface) effect);
        }
        catch (InvocationTargetException | IllegalAccessException | InstantiationException
        		| ClassNotFoundException | NoSuchMethodException e)
        {
            JOptionPane.showInternalMessageDialog(null, "Cannot initialize the effect",				//$NON-NLS-1$
            		"Error", JOptionPane.ERROR_MESSAGE);											//$NON-NLS-1$
            System.out.println("Exception in ExampleDelayTest::main: " + e);						//$NON-NLS-1$
            return;
        }

        // start playback.  With this, the mixer will obtain access to the output
        // audio device and will load it with some audio data
        if (! mixer.startPlay())
        {
            JOptionPane.showInternalMessageDialog(null, "Errors when attempting to play",		//$NON-NLS-1$
            		"Error", JOptionPane.ERROR_MESSAGE);										//$NON-NLS-1$
            return;
        }

        // continue playback.  The mixer will continue reading audio buffers and
        // sending them to the output audio device.  This is a separate thread so
        // that we can continue working with the application (in this case, with 
        // the EffectDialog.  As we make changes to the EffectDialog controls, these
        // changes will be heard during playback.  If we press the Close button of
        // the EffectDialog, playback will stop
        Thread thread = new Thread(mixer);
        thread.start();
	}
}
