package com.recordingblogs;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.net.URI;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Vector;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import javax.swing.*;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * Load the effects
 * 
 * Copyright (c) 2003 Anton Kamenov.  This file and the included source code is the 
 * property of Anton Kamenov.  Under no circumstances will the file and source code be 
 * sold, transferred, modified, or commercially exploited in part or in whole, without 
 * the prior written consent of Anton Kamenov.
 */

public class MainFrame extends JFrame
{
	/**
	 * This serial version UID is not used
	 */
	private static final long serialVersionUID = 7028317168034776736L;

	/**
	 * The one and only constructor.  Load the effects
	 */
	public MainFrame()
	{
		// nothing to do
	}

	/**
	 * Check if a file is a JAR file
	 * @param f The file to be tested
	 * @return true, if the file is a JAR file, false, if not a file or not a JAR file
	 */
    public static boolean isjar(File f)
    {
        if(f.isDirectory())
            return false;
        String extension = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');
        if(i > 0 && i < s.length() - 1)
            extension = s.substring(i + 1).toLowerCase();
        if(extension != null)
            return extension.equals("jar");			//$NON-NLS-1$
        return false;
    }

    /**
     * Load the effects
     */
	public void loadEffects()
	{
		// use this to store information about all effects found in the
		// /effects folder
        EffectStore.m_store = new Vector<>(0);
		
    	// look for jar files in the /effects directory.  If there are
		// none, exit
        File dir = new File("../orinj/effects/");													//$NON-NLS-1$
        File [] children = dir.listFiles();
        if (children == null) 
        {
            JOptionPane.showMessageDialog(null, "Cannot find effect packasge JARs",			//$NON-NLS-1$
            		"Error", JOptionPane.ERROR_MESSAGE);									//$NON-NLS-1$
            return;
        } 

        // loop through all files in the /effects folder.  Pick the JARs
        // look for effect.xml in the JARs, load the effects
        for (int i = 0; i < children.length; i++) 
        {
            // check whether the file is a JAR file
            if (isjar(children[i]))
            {
		        ZipEntry zipEntry = null;
		        ZipInputStream zipStream = null;
		        BufferedInputStream bufferStream = null;

		        // open the JAR file
            	try (JarFile jf = new JarFile(children[i].getPath());
            			FileInputStream fileStream = new FileInputStream(children[i].getPath()))
            	{
            		// keep this classloader for later
        			File file = new File(jf.getName());
        			URI uriclass = file.toURI();
        			URL [] urls = new URL [] {uriclass.toURL()};
        			@SuppressWarnings("resource")
					ClassLoader classloader = new URLClassLoader (urls);
            		
					// begin unzipping in search of effect.xml.  A JAR file is just
	            	// a zipped file

    		        bufferStream = new BufferedInputStream(fileStream);
    		        zipStream = new ZipInputStream(bufferStream);
    		        zipEntry = zipStream.getNextEntry();

	    			// loop through all items in the jar file and look for effect.xml
			        while ((zipEntry != null) && (! zipEntry.getName().equals("effect.xml"))) 	  //$NON-NLS-1$
			        {
    		            if (zipStream.available() == 0)
    		            	break;
        		        zipEntry = zipStream.getNextEntry();
			        }
    		    
			        // if we found effect.xml
		            if (zipEntry != null && zipEntry.getName().equals("effect.xml")) 	  		//$NON-NLS-1$
		            {
		            	// read the effect.xml file by parsing the information coming out
		            	// of the zip stream
            			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
            			Document doc = docBuilder.parse(zipStream);
	        		
		            	// get the contents of effect.xml according to the tags
	        			NodeList effectclass = doc.getElementsByTagName("effectclass"); 	  	//$NON-NLS-1$
	        			NodeList dialogclass = doc.getElementsByTagName("dialogclass"); 	  	//$NON-NLS-1$
	        			NodeList title = doc.getElementsByTagName("title"); 	  				//$NON-NLS-1$
	        			NodeList type = doc.getElementsByTagName("type"); 	  					//$NON-NLS-1$
	
	    				// loop through the effects and load them
	        			for(int j = 0; j < effectclass.getLength(); j++)
	        			{
	            			// load the effect class and the effect panel class
            				String filePath = children[i].getPath();
	            			URI uri = new File(filePath).toURI();
	            			URL [] url = new URL[1];
	            			url[0] = uri.toURL();
	            			
	            			// this is where the frame's class loader is used
	            			try (URLClassLoader child = new URLClassLoader (url, this.getClass().getClassLoader()))
	            			{
		            			Class<?> classToLoad = Class.forName(effectclass.item(j).getTextContent(), true, child);
		            			Constructor<?> [] ce = classToLoad.getConstructors();
		            			classToLoad = Class.forName(dialogclass.item(j).getTextContent(), true, child);
		            			Constructor<?> [] cd = classToLoad.getConstructors();
		            			
		            			// check that the size of ce and cd here is at least 1 and that
		            			//  they are not null.  The application bombs if one of the constructors is missing from the jar, which is possible
		            			//  if the whole class is missing.
		            			if (ce == null || cd == null)
		            			{
		    	    	            JOptionPane.showInternalMessageDialog(null, "Cannot find effects",				//$NON-NLS-1$
		    	    	            		"Error", JOptionPane.ERROR_MESSAGE);									//$NON-NLS-1$
		    	    	            return;
		            			}
		            			if (ce.length < 1 || cd.length < 1)
		            			{
		    	    	            JOptionPane.showInternalMessageDialog(null, "Mismatch when reading effects",	//$NON-NLS-1$
		    	    	            		"Error", JOptionPane.ERROR_MESSAGE);									//$NON-NLS-1$
		    	    	            return;
		            			}
		            			
	            				// there may be other things we would like to do here,
		            			// such as choose the right language for the effect, if the effect 
		            			// supports it.  For now, we will simply add the information to the 
		            			// effect store
	            				
		            			EffectStore.m_store.add(new EffectStoreItem(classloader, ce[0], cd[0], title.item(j).getTextContent(), type.item(j).getTextContent()));
	            			}
	            		}  // close the loop through the effects
	        		}
		            
		            // close the zip stream
       				zipStream.close();
       				bufferStream.close();
       				fileStream.close();

            	}
            	catch (IOException | SAXException | ParserConfigurationException | ClassNotFoundException e)
            	{
    	            JOptionPane.showInternalMessageDialog(null, "Exception when loading jar: "  //$NON-NLS-1$
    	            		+ children[i].getPath() + ": " + e,						  			//$NON-NLS-1$
    	            		"Error", JOptionPane.ERROR_MESSAGE);							  	//$NON-NLS-1$
            	}
            }  // end if statement on whether the file is a JAR file
        }  // end of loop through effect jar files
	}
}
